@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef KDECONNECTPLUGIN_H
@@ -23,12 +23,17 @@
 
 #include <QObject>
 #include <QVariantList>
+#include <kcoreaddons_version.h>
 
 #include "kdeconnectcore_export.h"
 #include "kdeconnectpluginconfig.h"
-#include "networkpackage.h"
+#include "networkpacket.h"
 #include "device.h"
 
+#if KCOREADDONS_VERSION < QT_VERSION_CHECK(5,44,0)
+#define K_PLUGIN_CLASS_WITH_JSON(classname,jsonFile) K_PLUGIN_FACTORY_WITH_JSON(classname ## Factory, jsonFile, registerPlugin<classname >();)
+#endif
+
 struct KdeConnectPluginPrivate;
 
 class KDECONNECTCORE_EXPORT KdeConnectPlugin
@@ -43,18 +48,20 @@ public:
     const Device* device();
     Device const* device() const;
 
-    bool sendPackage(NetworkPackage& np) const;
+    bool sendPacket(NetworkPacket& np) const;
 
     KdeConnectPluginConfig* config() const;
 
     virtual QString dbusPath() const;
 
+    QString iconName() const;
+
 public Q_SLOTS:
     /**
-     * Returns true if it has handled the package in some way
-     * device.sendPackage can be used to send an answer back to the device
+     * Returns true if it has handled the packet in some way
+     * device.sendPacket can be used to send an answer back to the device
      */
-    virtual bool receivePackage(const NetworkPackage& np) = 0;
+    virtual bool receivePacket(const NetworkPacket& np) = 0;
 
     /**
      * This method will be called when a device is connected to this computer.
