@@ -15,14 +15,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef DEVICESSORTPROXYMODEL_H
 #define DEVICESSORTPROXYMODEL_H
 
 #include <QSortFilterProxyModel>
-#include "interfaces/kdeconnectinterfaces_export.h"
+#include "kdeconnectinterfaces_export.h"
 
 class DevicesModel;
 
@@ -30,13 +30,9 @@ class KDECONNECTINTERFACES_EXPORT DevicesSortProxyModel : public QSortFilterProx
 {
     Q_OBJECT
 public:
-    explicit DevicesSortProxyModel(DevicesModel* devicesModel = Q_NULLPTR);
-    virtual bool lessThan(const QModelIndex& left, const QModelIndex& right) const override;
-    virtual bool filterAcceptsRow(int source_row, const QModelIndex& source_parent) const override;
-    virtual void setSourceModel(QAbstractItemModel *sourceModel) override;
-
-public Q_SLOTS:
-    void sourceDataChanged(QModelIndex,QModelIndex);
+    explicit DevicesSortProxyModel(DevicesModel* devicesModel = nullptr);
+    bool lessThan(const QModelIndex& left, const QModelIndex& right) const override;
+    bool filterAcceptsRow(int source_row, const QModelIndex& source_parent) const override;
 };
 
 #endif // DEVICESSORTPROXYMODEL_H
