@@ -15,20 +15,22 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "notificationsmodel.h"
 #include "interfaces_debug.h"
 
 #include <QDebug>
-#include <QDBusInterface>
-
-#include <KSharedConfig>
 #include <QIcon>
 
+#include <dbushelper.h>
+
 //#include "modeltest.h"
 
+//In older Qt released, qAsConst isnt available
+#include "core/qtcompat_p.h"
+
 NotificationsModel::NotificationsModel(QObject* parent)
     : QAbstractListModel(parent)
     , m_dbusInterface(nullptr)
@@ -47,7 +49,7 @@ NotificationsModel::NotificationsModel(QObject* parent)
             this, &NotificationsModel::anyDismissableChanged);
 
     QDBusServiceWatcher* watcher = new QDBusServiceWatcher(DaemonDbusInterface::activatedService(),
-                                                           QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this);
+                                                           DBusHelper::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this);
     connect(watcher, &QDBusServiceWatcher::serviceRegistered, this, &NotificationsModel::refreshNotificationList);
     connect(watcher, &QDBusServiceWatcher::serviceUnregistered, this, &NotificationsModel::clearNotifications);
 }
@@ -56,9 +58,9 @@ QHash<int, QByteArray> NotificationsModel::roleNames() const
 {
     //Role names for QML
     QHash<int, QByteArray> names = QAbstractItemModel::roleNames();
-    names.insert(DbusInterfaceRole,    "dbusInterface");
-    names.insert(AppNameModelRole,     "appName");
-    names.insert(IdModelRole,          "notificationId");
+    names.insert(DbusInterfaceRole, "dbusInterface");
+    names.insert(AppNameModelRole, "appName");
+    names.insert(IdModelRole, "notificationId");
     names.insert(DismissableModelRole, "dismissable");
     names.insert(RepliableModelRole, "repliable");
     names.insert(IconPathModelRole, "appIcon");
@@ -92,8 +94,6 @@ void NotificationsModel::setDeviceId(const QString& deviceId)
             this, &NotificationsModel::notificationRemoved);
     connect(m_dbusInterface, &OrgKdeKdeconnectDeviceNotificationsInterface::allNotificationsRemoved,
             this, &NotificationsModel::clearNotifications);
-    connect(m_dbusInterface, &OrgKdeKdeconnectDeviceNotificationsInterface::notificationUpdated,
-            this, &NotificationsModel::notificationUpdated);
 
     refreshNotificationList();
 
@@ -102,10 +102,10 @@ void NotificationsModel::setDeviceId(const QString& deviceId)
 
 void NotificationsModel::notificationAdded(const QString& id)
 {
-    int currentSize = m_notificationList.size();
-    beginInsertRows(QModelIndex(),  currentSize, currentSize);
+    beginInsertRows(QModelIndex(),  0, 0);
     NotificationDbusInterface* dbusInterface = new NotificationDbusInterface(m_deviceId, id, this);
-    m_notificationList.append(dbusInterface);
+    connect(dbusInterface, &NotificationDbusInterface::ready, this, &NotificationsModel::notificationUpdated);
+    m_notificationList.prepend(dbusInterface);
     endInsertRows();
 }
 
@@ -195,7 +195,7 @@ QVariant NotificationsModel::data(const QModelIndex& index, int role) const
         case AppNameModelRole:
             return notification->appName();
         case DbusInterfaceRole:
-            return qVariantFromValue<QObject*>(notification);
+            return QVariant::fromValue<QObject*>(notification);
         case DismissableModelRole:
             return notification->dismissable();
         case RepliableModelRole:
@@ -264,9 +264,7 @@ void NotificationsModel::clearNotifications()
     }
 }
 
-void NotificationsModel::notificationUpdated(const QString& id)
+void NotificationsModel::notificationUpdated()
 {
-    //TODO only emit the affected indices
-    Q_UNUSED(id);
     Q_EMIT dataChanged(index(0,0), index(m_notificationList.size() - 1, 0));
 }
