@@ -15,14 +15,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "mpriscontrolplugin.h"
 
 #include <QDBusArgument>
-#include <QDBusConnection>
-#include <QDBusInterface>
 #include <qdbusconnectioninterface.h>
 #include <QDBusReply>
 #include <QDBusMessage>
@@ -31,24 +29,37 @@
 #include <KPluginFactory>
 
 #include <core/device.h>
-#include "mprisdbusinterface.h"
-#include "propertiesdbusinterface.h"
+#include <dbushelper.h>
 
-K_PLUGIN_FACTORY_WITH_JSON( KdeConnectPluginFactory, "kdeconnect_mpriscontrol.json", registerPlugin< MprisControlPlugin >(); )
+#include "dbusproperties.h"
+#include "mprisplayer.h"
+#include "mprisroot.h"
+
+K_PLUGIN_CLASS_WITH_JSON(MprisControlPlugin, "kdeconnect_mpriscontrol.json")
 
 Q_LOGGING_CATEGORY(KDECONNECT_PLUGIN_MPRIS, "kdeconnect.plugin.mpris")
 
+
+MprisPlayer::MprisPlayer(const QString& serviceName, const QString& dbusObjectPath, const QDBusConnection& busConnection)
+    : m_serviceName(serviceName)
+    , m_propertiesInterface(new OrgFreedesktopDBusPropertiesInterface(serviceName, dbusObjectPath, busConnection))
+    , m_mediaPlayer2PlayerInterface(new OrgMprisMediaPlayer2PlayerInterface(serviceName, dbusObjectPath, busConnection))
+{
+    m_mediaPlayer2PlayerInterface->setTimeout(500);
+}
+
+
 MprisControlPlugin::MprisControlPlugin(QObject* parent, const QVariantList& args)
     : KdeConnectPlugin(parent, args)
     , prevVolume(-1)
 {
-    m_watcher = new QDBusServiceWatcher(QString(), QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this);
+    m_watcher = new QDBusServiceWatcher(QString(), DBusHelper::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this);
 
     // TODO: QDBusConnectionInterface::serviceOwnerChanged is deprecated, maybe query org.freedesktop.DBus directly?
-    connect(QDBusConnection::sessionBus().interface(), &QDBusConnectionInterface::serviceOwnerChanged, this, &MprisControlPlugin::serviceOwnerChanged);
+    connect(DBusHelper::sessionBus().interface(), &QDBusConnectionInterface::serviceOwnerChanged, this, &MprisControlPlugin::serviceOwnerChanged);
 
     //Add existing interfaces
-    const QStringList services = QDBusConnection::sessionBus().interface()->registeredServiceNames().value();
+    const QStringList services = DBusHelper::sessionBus().interface()->registeredServiceNames().value();
     for (const QString& service : services) {
         // The string doesn't matter, it just needs to be empty/non-empty
         serviceOwnerChanged(service, QLatin1String(""), QStringLiteral("1"));
@@ -58,8 +69,8 @@ MprisControlPlugin::MprisControlPlugin(QObject* parent, const QVariantList& args
 // Copied from the mpris2 dataengine in the plasma-workspace repository
 void MprisControlPlugin::serviceOwnerChanged(const QString& serviceName, const QString& oldOwner, const QString& newOwner)
 {
-    if (!serviceName.startsWith(QLatin1String("org.mpris.MediaPlayer2.")))
-        return;
+    if (!serviceName.startsWith(QStringLiteral("org.mpris.MediaPlayer2."))) return;
+    if (serviceName.startsWith(QStringLiteral("org.mpris.MediaPlayer2.kdeconnect."))) return;
 
     if (!oldOwner.isEmpty()) {
         qCDebug(KDECONNECT_PLUGIN_MPRIS) << "MPRIS service" << serviceName << "just went offline";
@@ -75,47 +86,72 @@ void MprisControlPlugin::serviceOwnerChanged(const QString& serviceName, const Q
 
 void MprisControlPlugin::addPlayer(const QString& service)
 {
-    QDBusInterface mprisInterface(service, QStringLiteral("/org/mpris/MediaPlayer2"), QStringLiteral("org.mpris.MediaPlayer2"));
-    //FIXME: This call hangs and returns an empty string if KDED is still starting!
-    QString identity = mprisInterface.property("Identity").toString();
+    const QString mediaPlayerObjectPath = QStringLiteral("/org/mpris/MediaPlayer2");
+
+    OrgMprisMediaPlayer2Interface iface(service, mediaPlayerObjectPath, DBusHelper::sessionBus());
+    QString identity = iface.identity();
+
     if (identity.isEmpty()) {
         identity = service.mid(sizeof("org.mpris.MediaPlayer2"));
     }
 
     QString uniqueName = identity;
-    for (int i = 1 ; !playerList[uniqueName].isEmpty() ; i++) {
-        uniqueName = identity + " [" + i + "]";
+    for (int i = 2; playerList.contains(uniqueName); ++i) {
+        uniqueName = identity + QLatin1String(" [") + QString::number(i) + QLatin1Char(']');
     }
 
-    playerList[uniqueName] = service;
+    MprisPlayer player(service, mediaPlayerObjectPath, DBusHelper::sessionBus());
+
+    playerList.insert(uniqueName, player);
+
+    connect(player.propertiesInterface(), &OrgFreedesktopDBusPropertiesInterface::PropertiesChanged,
+            this, &MprisControlPlugin::propertiesChanged);
+    connect(player.mediaPlayer2PlayerInterface(), &OrgMprisMediaPlayer2PlayerInterface::Seeked,
+            this, &MprisControlPlugin::seeked);
+
     qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Mpris addPlayer" << service << "->" << uniqueName;
     sendPlayerList();
-
-    OrgFreedesktopDBusPropertiesInterface* freedesktopInterface = new OrgFreedesktopDBusPropertiesInterface(service, QStringLiteral("/org/mpris/MediaPlayer2"), QDBusConnection::sessionBus(), this);
-    connect(freedesktopInterface, &OrgFreedesktopDBusPropertiesInterface::PropertiesChanged, this, &MprisControlPlugin::propertiesChanged);
-
-    OrgMprisMediaPlayer2PlayerInterface* mprisInterface0  = new OrgMprisMediaPlayer2PlayerInterface(service, QStringLiteral("/org/mpris/MediaPlayer2"), QDBusConnection::sessionBus());
-    connect(mprisInterface0, &OrgMprisMediaPlayer2PlayerInterface::Seeked, this, &MprisControlPlugin::seeked);
 }
 
 void MprisControlPlugin::seeked(qlonglong position){
     //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Seeked in player";
-    OrgFreedesktopDBusPropertiesInterface* interface = (OrgFreedesktopDBusPropertiesInterface*)sender();
-    const QString& service = interface->service();
-    const QString& player = playerList.key(service);
-
-    NetworkPackage np(PACKAGE_TYPE_MPRIS, {
-        {"pos", position/1000}, //Send milis instead of nanos
-        {"player", player}
+    OrgMprisMediaPlayer2PlayerInterface* mediaPlayer2PlayerInterface = (OrgMprisMediaPlayer2PlayerInterface*)sender();
+    const auto end = playerList.constEnd();
+    const auto it = std::find_if(playerList.constBegin(), end, [mediaPlayer2PlayerInterface](const MprisPlayer& player) {
+        return (player.mediaPlayer2PlayerInterface() == mediaPlayer2PlayerInterface);
     });
-    sendPackage(np);
+    if (it == end) {
+        qCWarning(KDECONNECT_PLUGIN_MPRIS) << "Seeked signal received for no longer tracked service" << mediaPlayer2PlayerInterface->service();
+        return;
+    }
+
+    const QString& playerName = it.key();
+
+    NetworkPacket np(PACKET_TYPE_MPRIS, {
+        {QStringLiteral("pos"), position/1000}, //Send milis instead of nanos
+        {QStringLiteral("player"), playerName}
+    });
+    sendPacket(np);
 }
 
 void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, const QVariantMap& properties)
 {
     Q_UNUSED(propertyInterface);
 
-    NetworkPackage np(PACKAGE_TYPE_MPRIS);
+    OrgFreedesktopDBusPropertiesInterface* propertiesInterface = (OrgFreedesktopDBusPropertiesInterface*)sender();
+    const auto end = playerList.constEnd();
+    const auto it = std::find_if(playerList.constBegin(), end, [propertiesInterface](const MprisPlayer& player) {
+        return (player.propertiesInterface() == propertiesInterface);
+    });
+    if (it == end) {
+        qCWarning(KDECONNECT_PLUGIN_MPRIS) << "PropertiesChanged signal received for no longer tracked service" << propertiesInterface->service();
+        return;
+    }
+
+    OrgMprisMediaPlayer2PlayerInterface* const mediaPlayer2PlayerInterface = it.value().mediaPlayer2PlayerInterface();
+    const QString& playerName = it.key();
+
+    NetworkPacket np(PACKET_TYPE_MPRIS);
     bool somethingToSend = false;
     if (properties.contains(QStringLiteral("Volume"))) {
         int volume = (int) (properties[QStringLiteral("Volume")].toDouble()*100);
@@ -130,7 +166,7 @@ void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, con
         QVariantMap nowPlayingMap;
         bullshit >> nowPlayingMap;
 
-        mprisPlayerMetadataToNetworkPackage(np, nowPlayingMap);
+        mprisPlayerMetadataToNetworkPacket(np, nowPlayingMap);
         somethingToSend = true;
     }
     if (properties.contains(QStringLiteral("PlaybackStatus"))) {
@@ -160,37 +196,87 @@ void MprisControlPlugin::propertiesChanged(const QString& propertyInterface, con
     }
 
     if (somethingToSend) {
-        OrgFreedesktopDBusPropertiesInterface* interface = (OrgFreedesktopDBusPropertiesInterface*)sender();
-        const QString& service = interface->service();
-        const QString& player = playerList.key(service);
-        np.set(QStringLiteral("player"), player);
+        np.set(QStringLiteral("player"), playerName);
         // Always also update the position
-        OrgMprisMediaPlayer2PlayerInterface mprisInterface(service, QStringLiteral("/org/mpris/MediaPlayer2"), QDBusConnection::sessionBus());
-        if (mprisInterface.canSeek()) {
-            long long pos = mprisInterface.position();
+        if (mediaPlayer2PlayerInterface->canSeek()) {
+            long long pos = mediaPlayer2PlayerInterface->position();
             np.set(QStringLiteral("pos"), pos/1000); //Send milis instead of nanos
         }
-        sendPackage(np);
+        sendPacket(np);
     }
 }
 
-void MprisControlPlugin::removePlayer(const QString& ifaceName)
+void MprisControlPlugin::removePlayer(const QString& serviceName)
 {
-    const QString identity = playerList.key(ifaceName);
-    qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Mpris removePlayer" << ifaceName << "->" << identity;
-    playerList.remove(identity);
+    const auto end = playerList.end();
+    const auto it = std::find_if(playerList.begin(), end, [serviceName](const MprisPlayer& player) {
+        return (player.serviceName() == serviceName);
+    });
+    if (it == end) {
+        qCWarning(KDECONNECT_PLUGIN_MPRIS) << "Could not find player for serviceName" << serviceName;
+        return;
+    }
+
+    const QString& playerName = it.key();
+    qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Mpris removePlayer" << serviceName << "->" << playerName;
+
+    playerList.erase(it);
+
     sendPlayerList();
 }
 
-bool MprisControlPlugin::receivePackage (const NetworkPackage& np)
+bool MprisControlPlugin::sendAlbumArt(const NetworkPacket& np)
+{
+    const QString player = np.get<QString>(QStringLiteral("player"));
+    auto it = playerList.find(player);
+    bool valid_player = (it != playerList.end());
+    if (!valid_player) {
+        return false;
+    }
+
+    //Get mpris information
+    auto& mprisInterface = *it.value().mediaPlayer2PlayerInterface();
+    QVariantMap nowPlayingMap = mprisInterface.metadata();
+
+    //Check if the supplied album art url indeed belongs to this mpris player
+    QUrl playerAlbumArtUrl{nowPlayingMap[QStringLiteral("mpris:artUrl")].toString()};
+    QString requestedAlbumArtUrl = np.get<QString>(QStringLiteral("albumArtUrl"));
+    if (!playerAlbumArtUrl.isValid() || playerAlbumArtUrl != QUrl(requestedAlbumArtUrl)) {
+        return false;
+    }
+
+    //Only support sending local files
+    if (playerAlbumArtUrl.scheme() != QStringLiteral("file")) {
+        return false;
+    }
+
+    //Open the file to send
+    QSharedPointer<QFile> art{new QFile(playerAlbumArtUrl.toLocalFile())};
+
+    //Send the album art as payload
+    NetworkPacket answer(PACKET_TYPE_MPRIS);
+    answer.set(QStringLiteral("transferringAlbumArt"), true);
+    answer.set(QStringLiteral("player"), player);
+    answer.set(QStringLiteral("albumArtUrl"), requestedAlbumArtUrl);
+    answer.setPayload(art, art->size());
+    sendPacket(answer);
+    return true;
+}
+
+bool MprisControlPlugin::receivePacket (const NetworkPacket& np)
 {
     if (np.has(QStringLiteral("playerList"))) {
         return false; //Whoever sent this is an mpris client and not an mpris control!
     }
 
+    if (np.has(QStringLiteral("albumArtUrl"))) {
+        return sendAlbumArt(np);
+    }
+
     //Send the player list
     const QString player = np.get<QString>(QStringLiteral("player"));
-    bool valid_player = playerList.contains(player);
+    auto it = playerList.find(player);
+    bool valid_player = (it != playerList.end());
     if (!valid_player || np.get<bool>(QStringLiteral("requestPlayerList"))) {
         sendPlayerList();
         if (!valid_player) {
@@ -199,38 +285,40 @@ bool MprisControlPlugin::receivePackage (const NetworkPackage& np)
     }
 
     //Do something to the mpris interface
-    OrgMprisMediaPlayer2PlayerInterface mprisInterface(playerList[player], QStringLiteral("/org/mpris/MediaPlayer2"), QDBusConnection::sessionBus());
-    mprisInterface.setTimeout(500);
+    const QString& serviceName = it.value().serviceName();
+    // turn from pointer to reference to keep the patch diff small,
+    // actual patch would change all "mprisInterface." into "mprisInterface->"
+    auto& mprisInterface = *it.value().mediaPlayer2PlayerInterface();
     if (np.has(QStringLiteral("action"))) {
         const QString& action = np.get<QString>(QStringLiteral("action"));
-        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Calling action" << action << "in" << playerList[player];
+        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Calling action" << action << "in" << serviceName;
         //TODO: Check for valid actions, currently we trust anything the other end sends us
         mprisInterface.call(action);
     }
     if (np.has(QStringLiteral("setVolume"))) {
         double volume = np.get<int>(QStringLiteral("setVolume"))/100.f;
-        qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Setting volume" << volume << "to" << playerList[player];
+        qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Setting volume" << volume << "to" << serviceName;
         mprisInterface.setVolume(volume);
     }
     if (np.has(QStringLiteral("Seek"))) {
         int offset = np.get<int>(QStringLiteral("Seek"));
-        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Seeking" << offset << "to" << playerList[player];
+        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Seeking" << offset << "to" << serviceName;
         mprisInterface.Seek(offset);
     }
 
     if (np.has(QStringLiteral("SetPosition"))){
         qlonglong position = np.get<qlonglong>(QStringLiteral("SetPosition"),0)*1000;
         qlonglong seek = position - mprisInterface.position();
-        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Setting position by seeking" << seek << "to" << playerList[player];
+        //qCDebug(KDECONNECT_PLUGIN_MPRIS) << "Setting position by seeking" << seek << "to" << serviceName;
         mprisInterface.Seek(seek);
     }
 
     //Send something read from the mpris interface
-    NetworkPackage answer(PACKAGE_TYPE_MPRIS);
+    NetworkPacket answer(PACKET_TYPE_MPRIS);
     bool somethingToSend = false;
     if (np.get<bool>(QStringLiteral("requestNowPlaying"))) {
         QVariantMap nowPlayingMap = mprisInterface.metadata();
-        mprisPlayerMetadataToNetworkPackage(answer, nowPlayingMap);
+        mprisPlayerMetadataToNetworkPacket(answer, nowPlayingMap);
 
         qlonglong pos = mprisInterface.position();
         answer.set(QStringLiteral("pos"), pos/1000);
@@ -251,9 +339,10 @@ bool MprisControlPlugin::receivePackage (const NetworkPackage& np)
         answer.set(QStringLiteral("volume"),volume);
         somethingToSend = true;
     }
+
     if (somethingToSend) {
         answer.set(QStringLiteral("player"), player);
-        sendPackage(answer);
+        sendPacket(answer);
     }
 
     return true;
@@ -261,20 +350,34 @@ bool MprisControlPlugin::receivePackage (const NetworkPackage& np)
 
 void MprisControlPlugin::sendPlayerList()
 {
-    NetworkPackage np(PACKAGE_TYPE_MPRIS);
+    NetworkPacket np(PACKET_TYPE_MPRIS);
     np.set(QStringLiteral("playerList"),playerList.keys());
-    sendPackage(np);
+    np.set(QStringLiteral("supportAlbumArtPayload"), true);
+    sendPacket(np);
 }
 
-void MprisControlPlugin::mprisPlayerMetadataToNetworkPackage(NetworkPackage& np, const QVariantMap& nowPlayingMap) const {
+void MprisControlPlugin::mprisPlayerMetadataToNetworkPacket(NetworkPacket& np, const QVariantMap& nowPlayingMap) const {
     QString title = nowPlayingMap[QStringLiteral("xesam:title")].toString();
     QString artist = nowPlayingMap[QStringLiteral("xesam:artist")].toString();
     QString album = nowPlayingMap[QStringLiteral("xesam:album")].toString();
     QString albumArtUrl = nowPlayingMap[QStringLiteral("mpris:artUrl")].toString();
-    QString nowPlaying = title;
-    if (!artist.isEmpty()) {
-        nowPlaying = artist + " - " + title;
+    QUrl fileUrl = nowPlayingMap[QStringLiteral("xesam:url")].toUrl();
+
+    if (title.isEmpty() && artist.isEmpty() && fileUrl.isLocalFile()) {
+        title = fileUrl.fileName();
+
+        QStringList splitUrl = fileUrl.path().split(QDir::separator());
+        if (album.isEmpty() && splitUrl.size() > 1) {
+            album = splitUrl.at(splitUrl.size() - 2);
+        }
     }
+
+    QString nowPlaying = title;
+
+    if (!artist.isEmpty()) {
+        nowPlaying = artist + QStringLiteral(" - ") + title;
+    }
+
     np.set(QStringLiteral("title"), title);
     np.set(QStringLiteral("artist"), artist);
     np.set(QStringLiteral("album"), album);
